<?php

namespace Slimdic\Test\Controller;

require_once 'Stub/StubController.php';

use Slim\Http\Environment;
use Slim\Http\Request;
use Slim\Http\Response;
use Slim\App;
use Slimdic\Controller\Factory;
use Chippyash\Type\String\StringType;

/**
 * Generated by PHPUnit_SkeletonGenerator on 2014-12-09 at 20:33:51.
 */
class FactoryTest extends \PHPUnit_Framework_TestCase
{

    protected $request;

    protected $response;

    protected $app;

    protected function setUp()
    {
        $this->request = Request::createFromEnvironment(Environment::mock());
        $this->response = new Response();
        $this->app = new App();
    }

    public function testControllerFactoryIsAbstract()
    {
        $refl = new \ReflectionClass('\Slimdic\Controller\Factory');
        $this->assertTrue($refl->isAbstract());
    }

    /**
     * @expectedException Exception
     * @expectedExceptionMessage Class Foo does not exist
     */
    public function testCreateThrowsExceptionIfClassDoesNotExist()
    {
        $ctrlName = new StringType('Foo');
        $ctrl = Factory::create($this->app, $this->request, $this->response, $ctrlName);
    }
    
    /**
     * @expectedException Exception
     * @expectedExceptionMessage stdClass is not an instance of AbstractController 
     */
    public function testCreatethrowsExceptionIfClassIsNotAController()
    {
        $ctrlName = new StringType('\stdClass');
        $ctrl = Factory::create($this->app, $this->request, $this->response, $ctrlName);
    }
    
    public function testCreateReturnsAControllerIfProperlyParameterised()
    {
        //we need to mock out the http $_SERVER for slim->request() to work
        global $_SERVER;
        $_SERVER = [
            'REQUEST_METHOD' => 'GET',
            'REMOTE_ADDR' => '127.0.0.1',
            'SCRIPT_NAME' => '/foo/index.php',
            'REQUEST_URI' => 'foo?a=b',
            'SERVER_NAME' => 'foo',
            'SERVER_PORT' => 80
        ];
        $ctrlName = new StringType('\Slimdic\Test\Controller\Stub\StubController');
        $ctrl = Factory::create($this->app, $this->request, $this->response, $ctrlName);
        $this->assertInstanceOf('\Slimdic\Controller\AbstractController', $ctrl);
    }

}
